module BulkraxHelper
  def rerun_failed_or_pending_entries_in_last_run(status="Failed")
    total_rerun_entries = 0
    current_run = @importer.last_run

    if current_run.failed_records.positive? || current_run.enqueued_records.positive?
      total_rerun_entries = rerun_failed_or_pending_entries(@importer, status)
    elsif current_run && @importer.parser.total.positive? && current_run.processed_records.zero?
      ::Bulkrax::ImporterJob.send(
        @importer.parser.perform_method,
        @importer.id
      )
    end

    total_rerun_entries
  end

  def rerun_failed_or_pending_entries(importer, status="Failed")
    current_run = importer.last_run
    total_rerun_entries = 0
    ::Bulkrax::ScheduleRelationshipsJob.set(wait: 5.minutes).perform_later(importer_id: importer.id)
    importer.status_info('Pending')

    importer.entries.each do |entry|
      next unless entry.status == status

      type = if entry.raw_metadata['model'] == 'CrcDataset'
               'Work'
             else
               entry.raw_metadata['model']
             end

      entry.status_info('Pending')
      if current_run.failed_records.positive?
        current_run.decrement!(:failed_records)
      end

      current_run.increment!(:enqueued_records)

      if type == 'Collection' && current_run.failed_collections.positive?
        current_run.decrement!(:failed_collections)
      elsif type == 'Work' && current_run.failed_works.positive?
        current_run.decrement!(:failed_works)
      elsif type == 'FileSet' && current_run.failed_file_sets.positive?
        current_run.decrement!(:failed_file_sets)
      elsif type == 'ComplexSubject' && current_run.failed_complex_subjects.positive?
        current_run.decrement!(:failed_complex_subjects)
      elsif type == 'ComplexSession' && current_run.failed_complex_sessions.positive?
        current_run.decrement!(:failed_complex_sessions)
      elsif type == 'ComplexModality' && current_run.failed_complex_modalities.positive?
        current_run.decrement!(:failed_complex_modalities)
      end

      "Bulkrax::CrcDataset::Import#{type}Job".constantize.send(
        entry.parser.perform_method,
        entry.id,
        importer.importer_runs.last.id
      )

      current_run.save
      total_rerun_entries +=1
    end

    total_rerun_entries
  end
end