module ComplexFilterHelper
  def fetch_crc_filter_params(params)
    facet_filters = params[:f].present? ? params[:f].slice(*::CrcDataset::CRC_FILTER_FACET_FIELDS).permit!.to_h : {}
    year_range_filters = (params[:range].present? && params[:range][:year_recorded_sim].present?) ? params[:range].slice(:year_recorded_sim).permit!.to_h : {}
    age_range_filters = (params[:range].present? && params[:range][:complex_subject_age_itsim].present?) ? params[:range].slice(:complex_subject_age_itsim).permit!.to_h : {}
    search_params = facet_filters.merge(year_range_filters).merge(age_range_filters)
  end

  def subject_field_mappings
    {"complex_subject_species_sim" => "subject_species", "complex_subject_type_sim" => "subject_type", "complex_subject_sex_sim" => "subject_sex", "complex_subject_age_itsim" => "subject_age"}
  end

  def session_field_mappings
    {"session_sim" => "session_title", "year_recorded_sim" => "session_date_recorded"}
  end

  def modality_field_mappings
    {"modality_sim" => "modality"}
  end

  def prepare_download_file(shell_file_path, doc_ids, search_params, parents_source_identifier, skip_filters= false)
    s3 = S3StorageService.new
    s3.init_client
    return s3.prepare_search_result_shell_file(doc_ids, shell_file_path) if skip_filters

    subject_query = {}
    session_query = {}
    modality_query = {}

    subject_query = build_query(subject_field_mappings, search_params).merge(parent_source_identifier: parents_source_identifier)
    session_query = build_query(session_field_mappings, search_params)
    modality_query = build_query(modality_field_mappings, search_params)

    if has_any_search_params?(modality_field_mappings, search_params)
      s3_bucket_and_object_prefix = filtered_modalities(subject_query, session_query, modality_query).map(&:s3_bucket_and_object_prifix)

      s3.prepare_search_result_shell_file(s3_bucket_and_object_prefix, shell_file_path)
    elsif has_any_search_params?(session_field_mappings, search_params)
      s3_bucket_and_object_prefix = filtered_sessions(subject_query, session_query).map(&:s3_bucket_and_object_prifix)
      s3.prepare_search_result_shell_file(s3_bucket_and_object_prefix, shell_file_path)
    elsif has_any_search_params?(subject_field_mappings, search_params)
      s3_bucket_and_object_prefix = filtered_subjects(subject_query).map(&:s3_bucket_and_object_prifix)

      s3.prepare_search_result_shell_file(s3_bucket_and_object_prefix, shell_file_path)
    else
      s3.prepare_search_result_shell_file(doc_ids, shell_file_path)
    end
  end

  def filtered_subjects(subject_query = nil, session_query = nil , modality_query = nil)
    complex_subjects = ComplexSubject.includes(complex_sessions: :complex_modalities)

    if modality_query.present?
      complex_subjects = complex_subjects.where(complex_modalities: modality_query)
    end

    if session_query.present?
      complex_subjects = complex_subjects.where(complex_sessions: session_query)
    end

    if subject_query.present?
      complex_subjects = complex_subjects.where(subject_query)
    end

    complex_subjects
  end

  def filtered_sessions(subject_query, session_query, modality_query)
    complex_sessions = ComplexSession.includes(:complex_subject, :complex_modalities)

    if subject_query.present?
      complex_sessions = complex_sessions.where(complex_subjects: subject_query)
    end

    if modality_query.present?
      complex_sessions = complex_sessions.where(complex_modalities: modality_query)
    end

    if session_query.present?
      complex_sessions = complex_sessions.where(session_query)
    end

    complex_sessions
  end

  def filtered_modalities(subject_query, session_query, modality_query)
    complex_modalities = ComplexModality.includes(complex_session: :complex_subject)

    if subject_query.present?
      complex_modalities = complex_modalities.where(complex_subjects: subject_query)
    end
  
    if session_query.present?
      complex_modalities = complex_modalities.where(complex_sessions: session_query)
    end
  
    if modality_query.present?
      complex_modalities = complex_modalities.where(modality_query)
    end

    complex_modalities
  end

  def build_query(field_mappings, search_params)
    field_mappings.each_with_object({}) do |(key, value), query|
      if search_params.key?(key)
        if key == "year_recorded_sim"
          year_range = search_params[key]
          if year_range.is_a?(Hash) && year_range.key?("begin") && year_range.key?("end")
            begin_date = Date.parse("#{year_range['begin']}-01-01")
            end_date = Date.parse("#{year_range['end']}-12-31")
            query["session_date_recorded"] = begin_date..end_date
          end
        elsif key == "complex_subject_age_itsim"
          age_range = search_params[key]
          if age_range.is_a?(Hash) && age_range.key?("begin") && age_range.key?("end")
            query["subject_age"] = age_range['begin']..age_range['end']
          else
            query[value] = search_params[key]
          end
        else
          query[value] = search_params[key]
        end
      end
    end
  end
  
  def has_any_search_params?(field_mappings, search_params)
    return field_mappings.any? { |key, _| search_params.key?(key) }
  end

  def has_any_crc_spacial_filter?(search_params)
    field_mappings = subject_field_mappings.merge(session_field_mappings).merge(modality_field_mappings)

    facet_filters = search_params[:f] || {}
    range_filters = (search_params[:range].present? && search_params[:range][:year_recorded_sim].present?) ? search_params[:range].slice(:year_recorded_sim) : {}
    age_range_filters = (params[:range].present? && params[:range][:complex_subject_age_itsim].present?) ? params[:range].slice(:complex_subject_age_itsim).permit!.to_h : {}
    search_params = facet_filters.merge(range_filters).merge(age_range_filters)

    return has_any_search_params?(field_mappings, search_params)
  end
end