require "./lib/vocabularies/rdms"
# frozen_string_literal: true
# Generated by hyrax:models:install
class FileSet < ActiveFedora::Base
  self.indexer = CrcFileSetIndexer
  include S3FileHandleable

  after_save :create_file_set_with_fast_load_meta_data
  before_destroy :destroy_set_meta_data

    property :for_complex_identifier, predicate: ::RDF::Vocab::Rdms.forComplexIdentifier, multiple: false  do |index|
    index.as :stored_searchable, :facetable
  end

  property :for_complex_type, predicate: ::RDF::Vocab::Rdms.forComplexType, multiple: false  do |index|
    index.as :stored_searchable, :facetable
  end
  include ::Hyrax::FileSetBehavior

  def fast_load_meta_data
    FileSetFastLoadMetadata.find_by(fedora_file_set_id: id)
  end

  def mime_type_image?
    file_extention = ".#{self.title.first&.split('.')&.last}"
    mime_type = Rack::Mime.mime_type(file_extention)
    mime_type.include?('image')
  end

  def object_key_on_s3
    return import_url if import_url.present?

    prefix = case for_complex_type
    when 'ComplexSubject'
       complex_subject = ComplexSubject.find_by(source_identifier: for_complex_identifier)
      "#{complex_subject.s3_folder_name}"
    when 'ComplexSession'
      complex_session = ComplexSession.find_by(source_identifier: for_complex_identifier)
      "#{complex_session.complex_subject.s3_folder_name}/#{complex_session.s3_folder_name}"
    when 'ComplexModality'
      complex_modality = ComplexModality.find_by(source_identifier: for_complex_identifier)
      "#{complex_modality.complex_subject.s3_folder_name}/#{complex_modality.complex_session.s3_folder_name}/#{complex_modality.s3_folder_name}"
    else
      nil
    end

    prefix.present? ? "#{prefix}/#{title.first}" : "#{title.first}"
  end


  def parent_object
    case for_complex_type
    when 'ComplexSubject'
      ComplexSubject.find_by(source_identifier: for_complex_identifier)
    when 'ComplexSession'
      ComplexSession.find_by(source_identifier: for_complex_identifier)
    when 'ComplexModality'
      ComplexModality.find_by(source_identifier: for_complex_identifier)
    else
      ActiveFedora::Base.find(parent_works.first.id)
    end
  end

  def in_works
    parent_ids = Hyrax.query_service.find_parents(resource: self).pluck(:id).map(&:to_s)
    ActiveFedora::Base.where(id: parent_ids)
  end

  private

  def create_file_set_with_fast_load_meta_data
    work_source_identifier = self.for_complex_identifier || self.parent_works.first&.source&.first
    work_type = self.for_complex_type || "CrcDataset"
    
    metadata = FileSetFastLoadMetadata.find_or_initialize_by(fedora_file_set_id: self.id)
    metadata.work_source_identifier = work_source_identifier
    metadata.work_model_name = work_type
    metadata.file_set_title = self.title&.first || self.label
    metadata.save

    # Here we are saving parent
    # We have callback before save that will set format and size here: ExternalServices#set_format_and_size
    # also update date_modified so download will not miss any files
    parent.save if parent.present?
  end

  def destroy_set_meta_data
    s3 = S3StorageService.new
    s3.init_client
    bucket_name = s3.sanitise_name(self.parent.id)
    meta_data = FileSetFastLoadMetadata.find_by(fedora_file_set_id: self.id)
    meta_data.destroy! if meta_data.present?

    s3.delete_object(bucket_name, self.object_key_on_s3)
    parent.update(date_modified: DateTime.now)
  end
end
